@@ -1,22 +1,25 @@
 #!/usr/bin/perl -wT
 #
-# $Id: FormMail.pl,v 1.66 2002/03/27 20:36:36 davorg Exp $
+# $Id: FormMail.pl,v 2.15 2002/09/26 22:10:09 nickjc Exp $
 #
 
 use strict;
-use POSIX qw(strftime);
+use POSIX qw(locale_h strftime);
+use Text::Wrap;              # Er for wrapping :)
 use Socket;                  # for the inet_aton()
 use CGI qw(:standard);
 use vars qw(
   $DEBUGGING $emulate_matts_code $secure
-  $allow_empty_ref $mailprog @referers @allow_mail_to
-  @recipients @valid_ENV $date_fmt $style
-  $send_confirmation_mail $confirmation_text
+  $allow_empty_ref $max_recipients $mailprog @referers
+  @allow_mail_to @recipients %recipient_alias
+  @valid_ENV $date_fmt $style $send_confirmation_mail
+  $confirmation_text $locale $charset $no_content
+  $double_spacing $wrap_text $wrap_style
 );
 
 # PROGRAM INFORMATION
 # -------------------
-# FormMail.pl $Revision: 1.66 $
+# FormMail.pl $Revision: 2.15 $
 #
 # This program is licensed in the same way as Perl
 # itself. You are free to choose between the GNU Public
@@ -40,14 +43,22 @@ BEGIN
   $DEBUGGING         = 1;
   $emulate_matts_code= 0;
   $secure            = 1;
-  $allow_empty_ref   = 0;
+  $allow_empty_ref   = 1;
+  $max_recipients    = 5;
   $mailprog          = '/usr/lib/sendmail -oi -t';
   @referers          = qw(dave.org.uk 209.207.222.64 localhost);
   @allow_mail_to     = qw(you@your.domain some.one.else@your.domain localhost);
   @recipients        = ();
+  %recipient_alias   = ();
   @valid_ENV         = qw(REMOTE_HOST REMOTE_ADDR REMOTE_USER HTTP_USER_AGENT);
+  $locale            = '';
+  $charset           = 'iso-8859-1';
   $date_fmt          = '%A, %B %d, %Y at %H:%M:%S';
   $style             = '/css/nms.css';
+  $no_content        = 0;
+  $double_spacing    = 1;
+  $wrap_text         = 0;
+  $wrap_style        = 1;
   $send_confirmation_mail = 0;
   $confirmation_text = <<'END_OF_CONFIRMATION';
 From: you@your.com
@@ -62,9 +73,12 @@ END_OF_CONFIRMATION
 # (no user serviceable parts beyond here)
 
   use vars qw($VERSION);
-  $VERSION = ('$Revision: 1.66 $' =~ /(\d+\.\d+)/ ? $1 : '?');
+  $VERSION = substr q$Revision: 2.15 $, 10, -1;
 
-  # Merge @allow_mail_to and @recipients into a single list of regexps
+  # Merge @allow_mail_to and @recipients into a single list of regexps,
+  # automatically adding any recipients in %recipient_alias.
+  push @allow_mail_to,
+     grep( /@/, split(/\s*,\s*/, join ',', values %recipient_alias) );
   push @recipients, map { /\@/ ? "^\Q$_\E\$" : "\@\Q$_\E\$" } @allow_mail_to;
 
   $secure = 0 if $emulate_matts_code;
@@ -104,14 +118,17 @@ BEGIN
          $message = '';
       }
 
-      my ( $pack, $file, $line, $sub ) = caller(1);
-      my ($id ) = $file =~ m%([^/]+)$%;
+      my ( $pack, $file ) = caller(0);
 
       return undef if $file =~ /^\(eval/;
 
-      print "Content-Type: text/html\n\n" unless $done_headers;
+      
+      $charset = 'iso-8859-1' unless $charset;
+
+      print "Content-Type: text/html; charset=$charset\n\n" unless $done_headers;
 
       print <<EOERR;
+<?xml version="1.0" encoding="$charset"?>
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
     "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
 <html xmlns="http://www.w3.org/1999/xhtml">
@@ -157,7 +174,7 @@ $CGI::POST_MAX        = 1000000;
 # general we have no way to tell what should be there.
 
 delete @ENV{qw(IFS CDPATH ENV BASH_ENV)};
-$ENV{PATH} =~ /(\S*)/ and $ENV{PATH} = $1;
+$ENV{PATH} =~ /(.*)/ and $ENV{PATH} = $1;
 
 
 use vars qw(%Config %Form);
@@ -166,6 +183,11 @@ use vars qw(%Config %Form);
 
 check_url();
 
+eval
+{
+   setlocale(LC_TIME, $locale) if $locale;
+};
+
 my $date = strftime($date_fmt, localtime);
 
 my @Field_Order = parse_form();
@@ -174,12 +196,17 @@ check_required();
 
 send_mail($date, [@Field_Order]);
 
-return_html($date, [@Field_Order]);
+if ( $no_content ) {
+   print header(-Status => 204);
+}
+else {
+   return_html($date, [@Field_Order]);
+}
 
 sub check_url {
-  my $check_referer = check_referer(referer());
-
-  error('bad_referer') unless $check_referer;
+  if ( scalar(@referers) and not check_referer(referer()) ) {
+    error('bad_referer');
+  }
 }
 
 sub check_referer
@@ -187,15 +214,14 @@ sub check_referer
   my $check_referer;
   my ($referer) = @_;
 
-  if ($referer && ($referer =~ m!^https?://([^/]*\@)?([^/]+)!i)) {
+  unless ($referer) {
+    return 1 if $allow_empty_ref or !$secure;
+  }
+
+  if ($referer && ($referer =~ m!^https?://([^/]*\@)?([\w\-\.]+)!i)) {
     my $refHost;
 
-    if (defined($1) and (! $secure)) {
-      $refHost = $1;
-      chop $refHost;
-    } else {
-      $refHost = $2;
-    }
+    $refHost = $2;
 
     foreach my $test_ref (@referers) {
       if ($refHost =~ m|\Q$test_ref\E$|i) {
@@ -216,7 +242,7 @@ sub check_referer
       }
     }
   } else {
-    $check_referer = ($secure && !$allow_empty_ref) ? 0 : 1;
+    return 0;
   }
 
   return $check_referer;
@@ -257,6 +283,7 @@ sub parse_form {
       next if /redirect$/ and not check_url_valid($val);
       next if /^return_link_url$/ and $secure and not check_url_valid($val);
       $Config{$_} = $val;
+      $Form{$_} = $val unless $emulate_matts_code;
     } else {
       my @vals = map {strip_nonprintable($_)} param($_);
       my $key = strip_nonprintable($_);
@@ -277,6 +304,14 @@ sub parse_form {
 
   $Config{env_report} = [ grep { $valid_ENV{$_} } @{$Config{env_report}} ];
 
+  if (defined $Config{'sort'}) {
+    if ($Config{'sort'} eq 'alphabetic') {
+      @field_order = sort @field_order;
+    } elsif ($Config{'sort'} =~ /^\s*order:\s*(.*)$/s) {
+      @field_order = split /\s*,\s*/, $1;
+    }
+  }
+
   return @field_order;
 }
 
@@ -285,14 +320,17 @@ sub check_required {
 
   defined $Config{subject} or $Config{subject} = '';
   defined $Config{recipient} or $Config{recipient} = '';
-  if ($Config{subject} =~ /[\n\r]/m ||
-      $Config{recipient} =~ /[\n\r]/m) {
-    error('no_recipient');
-  }
+  $Config{subject}   =~ s/[\r\n]+/ /g;
+  $Config{recipient} =~ s/[\r\n]+/ /g;
 
-  if ($Config{recipient}) {
+  if (length $Config{recipient}) {
     my @valid;
 
+    if (exists $recipient_alias{$Config{recipient}}) {
+      $Config{recipient} = $recipient_alias{$Config{recipient}};
+      $hide_recipient = 1;
+    }
+
     foreach (split /,/, $Config{recipient}) {
       next unless check_email($_);
 
@@ -302,6 +340,9 @@ sub check_required {
     }
 
     error('no_recipient') unless scalar @valid;
+    if ($max_recipients > 0 and not $emulate_matts_code) {
+      error('too_many_recipients') if scalar @valid > $max_recipients;
+    }
     $Config{recipient} = join ',', @valid;
 
   } else {
@@ -309,14 +350,12 @@ sub check_required {
     if (scalar @allow > 0 and not $emulate_matts_code) {
       $Config{recipient} = $allow[0];
       $hide_recipient = 1;
-    } elsif (%Form) {
-      error('no_recipient')
     } else {
-      error('bad_referer')
+      error('no_recipient')
     }
   }
 
-  if ($secure and request_method() ne 'POST') {
+  if ($secure and (! defined(request_method()) || request_method() ne 'POST')) {
     error('bad_method');
   }
 
@@ -348,12 +387,11 @@ sub check_recipient {
 
 sub return_html {
   my ($date, $Field_Order) = @_;
-  my ($key, $sort_order, $sorted_field);
 
   if ($Config{'redirect'}) {
     print redirect $Config{'redirect'};
   } else {
-    print header();
+    print "Content-Type: text/html; charset=$charset\n\n";
     $done_headers++;
 
     my $title = escape_html( $Config{'title'} || 'Thank You' );
@@ -362,6 +400,7 @@ sub return_html {
     my $attr = body_attributes(); # surely this should be done with CSS
 
     print <<EOHTML;
+<?xml version="1.0" encoding="$charset"?>
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
     "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
 <html xmlns="http://www.w3.org/1999/xhtml">
@@ -380,27 +419,11 @@ sub return_html {
     <p><hr size="1" width="75%" /></p>
 EOHTML
 
-    my @sorted_fields;
-    if ($Config{'sort'}) {
-      if ($Config{'sort'} eq 'alphabetic') {
-        @sorted_fields = sort keys %Form;
-      } elsif ($Config{'sort'} =~ /^order:.*,.*/) {
-        $sort_order = $Config{'sort'};
-        $sort_order =~ s/(\s+|\n)?,(\s+|\n)?/,/g;
-        $sort_order =~ s/(\s+)?\n+(\s+)?//g;
-        $sort_order =~ s/order://;
-        @sorted_fields = split(/,/, $sort_order);
-      } else {
-        @sorted_fields = @$Field_Order;
-      }
-    } else {
-      @sorted_fields = @$Field_Order;
-    }
-
-    foreach (@sorted_fields) {
-      if ($Config{print_blank_fields} || $Form{$_} !~ /^\s*$/) {
+    foreach (@$Field_Order) {
+      my $val = (defined $Form{$_} ? $Form{$_} : '');
+      if ($Config{print_blank_fields} || $val !~ /^\s*$/) {
         print '<p><b>', escape_html($_), ':</b> ',
-                        escape_html($Form{$_}), "</p>\n";
+                        escape_html($val), "</p>\n";
       }
     }
 
@@ -454,21 +477,43 @@ sub send_mail {
   }
 
   my $xheader = '';
+
+  # This is more lenient than that in check_referer() because we
+  # want to know how people got this far if they are faking it
+  # however it is probably prudent to restrict to the characters
+  # valid in a URL - or what ?
+
+  if ( $secure and defined (my $referer = referer()) ) {
+    if ( $referer =~ /([\d\w.:@&%\/;?,-]{1,128})/ ) {
+       $xheader .= "X-HTTP-Referer: [$1]\n";
+    }
+  }
+
+  # however if remote_addr() is not pukka then something
+  # really bad is going on here.
+
   if ( $secure and defined (my $addr = remote_addr()) ) {
-    $addr =~ /^([\d\.]+)$/ or die "bad remote addr [$addr]";
-    $xheader = "X-HTTP-Client: [$1]\n"
-             . "X-Generated-By: NMS FormMail.pl v$VERSION\n";
+    $addr =~ /^\[?([\d\.]+)\]?$/ or die "bad remote addr [$addr]";
+
+    $addr = $1;
+
+    # The actual name of the program could be useful if there is
+    # more than one FormMail on the machine with different names.
+
+    my ( $realagent ) = $0 =~ m%([\d\w.]+)$%;
+    $realagent = defined $realagent ? "($realagent)" : '';
+    $xheader .= "X-HTTP-Client: [$addr]\n"
+             . "X-Mailer: NMS FormMail.pl $realagent v$VERSION"
+             . "[http://nms-cgi.sourceforge.net/]\n";
   }
 
   if ( $send_confirmation_mail ) {
-    open(CMAIL,"|$mailprog")
-      || die "Can't open $mailprog\n";
+    open_sendmail_pipe(\*CMAIL, $mailprog);
     print CMAIL $xheader, "To: $email$realname\n$confirmation_text";
     close CMAIL;
   }
 
-  open(MAIL,"|$mailprog")
-    || die "Can't open $mailprog\n";
+  open_sendmail_pipe(\*MAIL, $mailprog);
 
   print MAIL $xheader, <<EOMAIL;
 To: $Config{recipient}
@@ -478,35 +523,34 @@ Subject: $subject
 Below is the result of your feedback form.  It was submitted by
 $Config{realname} (${\( $Config{email}||'' )}) on $date
 $dashes
-
-
 EOMAIL
 
+  print MAIL "\n\n" if $double_spacing;
+  my $nl = ( $double_spacing ? "\n\n" : "\n" );
+
   if ($Config{print_config}) {
     foreach (@{$Config{print_config}}) {
-      print MAIL "$_: $Config{$_}\n\n" if $Config{$_};
+      print MAIL "$_: $Config{$_}$nl" if $Config{$_};
     }
   }
 
-  my @sorted_keys;
-  if ($Config{'sort'}) {
-    if ($Config{'sort'} eq 'alphabetic') {
-      @sorted_keys = sort keys %Form;
-    } elsif ($Config{'sort'} =~ /^order:.*,.*/) {
-      $Config{'sort'} =~ s/(\s+|\n)?,(\s+|\n)?/,/g;
-      $Config{'sort'} =~ s/(\s+)?\n+(\s+)?//g;
-      $Config{'sort'} =~ s/order://;
-      @sorted_keys = split(/,/, $Config{'sort'});
-    } else {
-      @sorted_keys = @$Field_Order;
-    }
-  } else {
-    @sorted_keys = @$Field_Order;
-  }
-
-  foreach (@sorted_keys) {
-    if ($Config{'print_blank_fields'} || $Form{$_} !~ /^\s*$/) {
-      print MAIL "$_: $Form{$_}\n\n";
+  foreach (@{$Field_Order}) {
+    my $val = (defined $Form{$_} ? $Form{$_} : '');
+    if ($Config{'print_blank_fields'} || $val !~ /^\s*$/) {
+      my $field_name = "$_: ";
+      if ( $wrap_text and length("$field_name$val") > 72 ) {
+        my $subs_indent = '';
+        if ( $wrap_style == 1 ) {
+          $subs_indent = ' ' x length($field_name);
+        }
+        $Text::Wrap::columns = 72;
+        my $wraped;
+        eval { local $SIG{__DIE__} ; $wraped = wrap($field_name,$subs_indent,$val) };
+        print MAIL +($@ ? "$field_name$val" : $wraped), $nl;
+      }
+      else {
+        print MAIL "$field_name$val$nl";
+      }
     }
   }
 
@@ -519,6 +563,22 @@ EOMAIL
   close (MAIL) || die "close mailprog: \$?=$?,\$!=$!";
 }
 
+sub open_sendmail_pipe {
+  my ($fh, $mailprog) = @_;
+
+  my $result;
+  eval { local $SIG{__DIE__};
+         $result = open $fh, "| $mailprog"
+       };
+  if ($@) {
+    die $@ unless $@ =~ /Insecure directory/;
+    delete $ENV{PATH};
+    $result = open $fh, "| $mailprog";
+  }
+
+  die "Can't open $mailprog\n" unless $result;
+}
+
 sub cleanup_realname {
   my ($realname) = @_;
 
@@ -530,7 +590,7 @@ sub cleanup_realname {
     # Allow no unusual characters and impose a length limit. We
     # need to allow extended ASCII characters because they can
     # occur in non-English names.
-    $realname =~ tr# a-zA-Z0-9_\-,./'\200-377##dc;
+    $realname =~ tr# a-zA-Z0-9_\-,./'\200-\377# #cs;
     $realname = substr $realname, 0, 128;
   } else {
     # Be as generous as possible without opening any known or
@@ -630,7 +690,23 @@ sub check_url_valid {
 sub strip_nonprintable {
   my $text = shift;
   return '' unless defined $text;
-  $text=~ tr#\011\012\040-\176\200-\377##dc;
+  if ($charset =~ /^iso-8859/i)
+  {
+    # None of the the iso-8859-* charsets have printable
+    # characters between \200 and \241.  See
+    # http://czyborra.com/charsets/iso8859.html
+    $text=~ tr#\t\n\040-\176\241-\377# #cs;
+  }
+  elsif ($charset =~ /^utf-8$/i)
+  {
+    # The bytes 0xFE and 0xFF are illegal in UTF-8, see
+    # http://www.cl.cam.ac.uk/~mgk25/unicode.html#utf-8
+    $text=~ tr#\t\n\040-\176\200-\375# #cs;
+  }
+  else
+  {
+    $text=~ tr#\t\n\040-\176\200-\377# #cs;
+  }
   return $text;
 }
 
@@ -725,7 +801,7 @@ EOBODY
   If you are attempting to configure this form to run with FormMail,
   you need to set the request method to POST in the opening form tag,
   like this:
-  <tt>&lt;form action=&quot;/cgi-bin/FormMail.pl&quot; method=&quot;POST&quot;&gt;</tt>
+  <tt>&lt;form action=&quot;/cgi-bin/FormMail.pl&quot; method=&quot;post&quot;&gt;</tt>
 </p>
 EOBODY
  } elsif ($error eq 'no_recipient') {
@@ -745,6 +821,18 @@ EOBODY
 <p>
  The recipient was: [ $recipient ]
 </p>
+EOBODY
+  }
+  elsif ( $error eq 'too_many_recipients' ) {
+    $title = 'Error: Too many Recipients';
+    $error_body =<<EOBODY;
+<p>
+  The number of recipients configured in the form exceeds the
+  maximum number of recipients configured in the script.  If
+  you are attempting to configure FormMail to run with this form
+  then you will need to increase the <tt>\$max_recipients</tt>
+  configuration setting in the script.
+</p>
 EOBODY
   }
   elsif ( $error eq 'missing_fields' ) {
@@ -799,6 +887,7 @@ EOBODY
               text-align: center;
             }
        th.c1 {
+               text-align: center;
                font-size: 143%;
              }
        p.c3 {font-size: 80%; text-align: center}
@@ -807,13 +896,11 @@ EOBODY
     </style>
   </head>
   <body>$debug_warnings
-    <table border="0" width="600" bgcolor="#9C9C9C" align="center" summary="">
-      <tr>
+    <table border="0" width="600" bgcolor="#9C9C9C" summary="">
+      <tr bgcolor="#9C9C9C">
         <th class="c1">$title</th>
       </tr>
-    </table>
-    <table border="0" width="600" bgcolor="#CFCFCF">
-      <tr>
+      <tr bgcolor="#CFCFCF">
         <td>
           $error_body
           <hr size="1" />
@@ -859,7 +946,7 @@ sub escape_html {
 
 =head1 COPYRIGHT
 
-FormMail $Revision: 1.66 $
+FormMail $Revision: 2.15 $
 Copyright 2001 London Perl Mongers, All rights reserved
 
 =head1 LICENSE
@@ -879,7 +966,7 @@ HTML form submission via an email message.
 
 =head1 FILES
 
-In this distribution, you will find three files:
+In this distribution, you will find the following files:
 
 =over
 
@@ -892,6 +979,14 @@ The main Perl script
 This documentation. Instructions on how to install and use
 formmail
 
+=item EXAMPLES
+
+Some worked examples of ways to set up formmail
+
+=item ChangeLog
+
+The change history of these files
+
 =item MANIFEST
 
 List of files
@@ -901,7 +996,7 @@ List of files
 
 =head1 CONFIGURATION
 
-There are a number of variables that you can change in formmail.pl which
+There are a number of variables that you can change in FormMail.pl which
 alter the way that the program works.
 
 =over
@@ -932,7 +1027,7 @@ do not recommend changing this variable to 0, as the
 resulting drop in security may leave your formmail
 open to use as a SPAM relay.
 
-=item allow_empty_ref
+=item $allow_empty_ref
 
 Some web proxies and office firewalls may strip
 certain headers from the HTTP request that is sent
@@ -946,6 +1041,16 @@ complaining about requests where no referer header
 was sent while leaving the rest of the security
 features intact.
 
+=item $max_recipients
+
+The maximum number of e-mail addresses that any single
+form should be allowed to send copies of the e-mail to.
+If none of your forms send e-mail to more than one
+recipient, then we recommend that you improve the
+security of FormMail by reducing this value to 1.
+Setting this variable to 0 removes all limits on the
+number of recipients of each e-mail.
+
 =item $mailprog
 
 The system command that the script should invoke to
@@ -957,6 +1062,22 @@ requires should be provided here. Your hosting
 provider or system administrator should be able to
 tell you what to set this variable to.
 
+A $mailprog setting that works for many UNIX-like
+hosts is:
+
+  $mailprog = '/usr/lib/sendmail -oi -t';
+
+Some other UNIX-like hosts need: 
+
+  $mailprog = '/usr/sbin/sendmail -oi -t';
+
+For hosts that lack a suitable sendmail binary (such
+as most Windows systems) we have a Perl script which
+does the job of the sendmail binary, in the nms_sendmail
+package at E<lt>http://nms-cgi.sourceforge.net/E<gt>.
+See the README file in the nms_sendmail package for
+instructions.
+
 =item @referers
 
 A list of referring hosts. This should be a list of
@@ -1002,6 +1123,73 @@ If that last paragraph makes no sense to you then
 please don't put anything in @recipients, stick to
 using the less error prone @allow_mail_to.
 
+=item %recipient_alias
+
+A hash for predefining a list of recipients in the script,
+and then choosing between them using the recipient form
+field, while keeping all the email addresses out of the
+HTML so that they don't get collected by address
+harvesters and sent junk email.
+
+For example, suppose you have three forms on your site,
+and you want each to submit to a different email address
+and you want to keep the addresses hidden.  You might
+set up C<%recipient_alias> like this:
+
+  %recipient_alias = (
+                       '1' => 'one@your.domain',
+                       '2' => 'two@your.domain',
+                       '3' => 'three@your.domain',
+                     );
+
+In the HTML form that should submit to the recipient
+'two@your.domain', you would then set the recipient with:
+
+  <input type="hidden" name="recipient" value="2" />
+
+=item $locale
+
+This determines the language that is used in the date - by
+default this is blank and the language will probably be
+english. The following a list of some possible values,
+however it should be stressed that not all of these will
+be supported on all systems and also this is not a complete
+list:
+
+        Catalan           ca_ES
+        Croatian          hr_HR
+        Czech             cs_CZ
+        Danish            da_DK
+        Dutc              nl_NL
+        Estonian          et_EE
+        Finnish           fi_FI
+        French            fr_FR
+        Galician          gl_ES
+        German            de_DE
+        Greek             el_GR
+        Hebrew            he_IL
+        Hungarian         hu_HU
+        Icelandic         is_IS
+        Italian           it_IT
+        Japanese          ja_JP
+        Korean            ko_KR
+        Lithuanian        lt_LT
+        Norwegian         no_NO
+        Polish            pl_PL
+        Portuguese        pt_PT
+        Romanian          ro_RO
+        Russian           ru_RU
+        Slovak            sk_SK
+        Slovenian         sl_SI
+        Spanish           es_ES
+        Swedish           sv_SE
+        Thai              th_TH
+        Turkish           tr_TR
+
+=item $charset
+
+The character set to use for output documents.
+
 =item @valid_ENV
 
 A list of all the environment variables that you want
@@ -1034,8 +1222,8 @@ part of the date or time. Here are some common tags:
 =item $style
 
 This is the URL of a CSS stylesheet which will be
-used for script generated messages.  This probably
-wants to be the same as the one that you use for all
+used for script generated messages.  This should
+probably be the same as the one that you use for all
 the other pages.  This should be a local absolute URI
 fragment.  Set $style to '0' or the empty string if
 you do not want to use style sheets.
@@ -1062,7 +1250,7 @@ Perl 'here document' to allow us to configure it
 as a single block of text in the script. In the
 example below, everything between the lines
 
-  $confirmation_text = E<lt>E<lt>'END_OF_CONFIRMATION';
+  $confirmation_text = <<'END_OF_CONFIRMATION';
 
 and
 
@@ -1101,7 +1289,7 @@ To make use of it, you need to write an HTML form that refers to the
 FormMail script. Here's an example which will send mail to the address
 'feedback@your.domain' when someone submits the form:
 
-  <form method="POST" action="http://your.domain/cgi-bin/FormMail.pl">
+  <form method="post" action="http://your.domain/cgi-bin/FormMail.pl">
     <input type="hidden" name="recipient" value="feedback@your.domain" />
     <input type="text" name="feedback" /><br />
     Please enter your comments<br />
